<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\Util\Functions;

class RelManutContaClieConsFamController {
    protected $utilService;
    protected $formatBirt;

    const EXTRATO = "extrato";

    public function __construct($entityManager, Utilities\Util $utilService, BirtReport $birtReport, ReportService $reportService) {
        $this->entityManager = $entityManager;
        $this->utilService   = $utilService;
        $this->birtReport    = $birtReport;
        $this->reportService = $reportService;
    }

    public function findRelatorio(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        if($row['TIPORELATORIO'] == RelManutContaClieConsFamController::EXTRATO) {
            if($this->validateConditions($row)) {
                if(!isset($row['FORMAT'])) {
                    $this->formatBirt = 'pdf';
                } else {
                    $this->formatBirt = $row['FORMAT'];
                }
                
                $params = $this->paramBuild($row);
    
                $this->getReport($params, $response);
            } else {
                $response->setError(new Error('Erro ao encontrar o relatório', 500));
            }
        } else {
            $response->setError(new Error('Erro ao encontrar o relatório', 500));
        }
    }

    private function validateConditions($conditions) {
        if((isset($conditions['CDCLIENTE'])) && (!empty($conditions['CDCLIENTE'])) &&
           (isset($conditions['DTPERIODO'])) && (!empty($conditions['DTPERIODO']))) {
            return true;
        } else {
            return false;
        }
    }

    private function paramBuild($conditions) {
        $params = [
            'ReportName'    => $conditions['TIPORELATORIO'],
            'ParametersRow' => [
                'P_CDCLIENTE'    => $conditions['CDCLIENTE'],
                'P_DTEXTRATOINI' => $conditions['DTPERIODO']['start'],
                'P_DTEXTRATOFIN' => $conditions['DTPERIODO']['end']
            ]
        ];
        
        // CDTIPOCONS
        if(!empty($conditions['CDTIPOCONS'])) {
            $params['ParametersRow']['P_CDTIPOCONS']  = '_'.implode('_', $conditions['CDTIPOCONS']).'_';
            $params['ParametersRow']['P_TCDTIPOCONS'] = "F";
        } else {
            $params['ParametersRow']['P_CDTIPOCONS']  = "";
            $params['ParametersRow']['P_TCDTIPOCONS'] = "T";
        }
        
        // CDFAMILISALD
        if(!empty($conditions['CDFAMILISALD'])) {
            $params['ParametersRow']['P_CDFAMILISALD']  = '_'.implode('_', $conditions['CDFAMILISALD']).'_';
            $params['ParametersRow']['P_TCDFAMILISALD'] = "F";
        } else {
            $params['ParametersRow']['P_CDFAMILISALD']  = "";
            $params['ParametersRow']['P_TCDFAMILISALD'] = "T";
        }

        // CDCONSUMIDOR
        if(!empty($conditions['CDCONSUMIDOR'])) {
            $params['ParametersRow']['P_CDCONSUMIDOR']  = '_'.implode('_', $conditions['CDCONSUMIDOR']).'_';
            $params['ParametersRow']['P_TCDCONSUMIDOR'] = "F";
        } else {
            $params['ParametersRow']['P_CDCONSUMIDOR']  = "";
            $params['ParametersRow']['P_TCDCONSUMIDOR'] = "T";
        }

        return $params;
    }

    public function getReport($params, DTO\Response $response) {
        try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $params['ReportName'], $params['ParametersRow'], true);
            
            if($this->formatBirt === 'xls') {
                $birtURL = str_replace('pdf', 'xls', $birtURL);
            }

            $response->addDataset(new DataSet('URL', array('url' => $birtURL)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}